==> yew-hooks-0.1.9.crate: yew-hooks-0.1.9/src/hooks/use_timeout.rs <==
use gloo::timers::callback::Timeout;
use yew::{use_effect_with_deps, use_mut_ref};

/// A hook that schedules a timeout to invoke `callback` in `millis` milliseconds from now.
/// The timeout will be cancelled if `millis` is set to 0.
///
/// # Example
///
/// ```rust
/// # use yew::prelude::*;
/// #
/// # use yew_hooks::use_timeout;
/// #
/// #[function_component(Timeout)]
/// fn timeout() -> Html {
///     let state = use_state(|| 0);
///
///     {
///         let state = state.clone();
///         use_timeout(move || {
///             state.set(1);
///         }, 2000);
///     }
///     
///     html! {
///         <>
///             { *state }
///         </>
///     }
/// }
/// ```
pub fn use_timeout<Callback>(callback: Callback, millis: u32)
where
    Callback: FnOnce() + 'static,
{
    let callback_ref = use_mut_ref(|| None);
    let callback_buffer_ref = use_mut_ref(|| None);
    let timeout_ref = use_mut_ref(|| None);

    // Update the ref each render so if it changes the newest callback will be invoked.
    *callback_ref.borrow_mut() = Some(callback);

    use_effect_with_deps(
        move |millis| {
            if *millis > 0 {
                *timeout_ref.borrow_mut() = Some(Timeout::new(*millis, move || {
                    if let Some(callback) = (*callback_ref.borrow_mut()).take() {
                        *callback_buffer_ref.borrow_mut() = Some(callback);
                    }

                    if let Some(callback) = (*callback_buffer_ref.borrow_mut()).take() {
                        callback();
                    }
                }));
            } else {
                *timeout_ref.borrow_mut() = None;
            }

            move || *timeout_ref.borrow_mut() = None
        },
        millis,
    );
}

==> yew-hooks-0.1.11.crate: yew-hooks-0.1.11/src/hooks/use_timeout.rs <==
use gloo::timers::callback::Timeout;
use yew::{use_effect_with_deps, use_mut_ref};

/// A hook that schedules a timeout to invoke `callback` in `millis` milliseconds from now.
/// The timeout will be cancelled if `millis` is set to 0.
///
/// # Example
///
/// ```rust
/// # use yew::prelude::*;
/// #
/// # use yew_hooks::use_timeout;
/// #
/// #[function_component(Timeout)]
/// fn timeout() -> Html {
///     let state = use_state(|| 0);
///
///     {
///         let state = state.clone();
///         use_timeout(move || {
///             state.set(1);
///         }, 2000);
///     }
///     
///     html! {
///         <>
///             { *state }
///         </>
///     }
/// }
/// ```
pub fn use_timeout<Callback>(callback: Callback, millis: u32)
where
    Callback: FnOnce() + 'static,
{
    let callback_ref = use_mut_ref(|| None);
    let timeout_ref = use_mut_ref(|| None);

    // Update the ref each render so if it changes the newest callback will be invoked.
    *callback_ref.borrow_mut() = Some(callback);

    use_effect_with_deps(
        move |millis| {
            if *millis > 0 {
                *timeout_ref.borrow_mut() = Some(Timeout::new(*millis, move || {
                    let callback = (*callback_ref.borrow_mut()).take();

                    if let Some(callback) = callback {
                        callback();
                    }
                }));
            } else {
                *timeout_ref.borrow_mut() = None;
            }

            move || *timeout_ref.borrow_mut() = None
        },
        millis,
    );
}
